@@ -19,7 +19,7 @@ from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, thgrepo
 
 PANEL_DEFAULT = ('rev', 'summary', 'user', 'dateage', 'branch', 'close',
-                 'tags', 'transplant', 'p4', 'svn')
+                 'tags', 'transplant', 'p4', 'svn', 'converted')
 
 def create(repo, target=None, style=None, custom=None, **kargs):
     return Factory(repo, custom, style, target, **kargs)()
@@ -110,7 +110,7 @@ class SummaryInfo(object):
               'tags': _('Tags:'), 'rawbranch': _('Branch:'),
               'rawtags': _('Tags:'), 'transplant': _('Transplant:'),
               'p4': _('Perforce:'), 'svn': _('Subversion:'),
-              'shortuser': _('User:')}
+              'converted': _('Converted From:'), 'shortuser': _('User:')}
 
     def __init__(self):
         pass
@@ -201,6 +201,13 @@ class SummaryInfo(object):
                     return cvt.split('@')[-1]
                 else:
                     return None
+            elif item == 'converted':
+                extra = ctx.extra()
+                cvt = extra.get('convert_revision', '')
+                if cvt and not cvt.startswith('svn:'):
+                    return cvt
+                else:
+                    return None
             elif item == 'ishead':
                 childbranches = [cctx.branch() for cctx in ctx.children()]
                 return ctx.branch() not in childbranches
@@ -249,7 +256,7 @@ class SummaryInfo(object):
                 return '%s' % revid
             elif item in ('revid', 'transplant'):
                 return qtlib.markup(value, **mono)
-            elif item in ('revnum', 'p4', 'close'):
+            elif item in ('revnum', 'p4', 'close', 'converted'):
                 return str(value)
             elif item == 'svn':
                 # svn is always in utf-8 because ctx.extra() isn't converted
