@@ -32,7 +32,7 @@ class ChangesetList(QWidget):
         self.lstyle = csinfo.labelstyle(contents=contents, width=350,
                                         selectable=True)
         contents = ('item', 'summary', 'user', 'dateage', 'rawbranch',
-                    'tags', 'transplant', 'p4', 'svn')
+                    'tags', 'transplant', 'p4', 'svn', 'converted')
         self.pstyle = csinfo.panelstyle(contents=contents, width=350,
                                         selectable=True)
 
