@@ -309,6 +309,9 @@ class HgFileView(QFrame):
         self.actionNextDiff.setEnabled(False)
         self.actionPrevDiff.setEnabled(False)
 
+        self.maxWidth = 0
+        self.sci.showHScrollBar(False)
+
     def displayFile(self, filename=None, status=None):
         if isinstance(filename, (unicode, QString)):
             filename = hglib.fromunicode(filename)
@@ -434,6 +437,20 @@ class HgFileView(QFrame):
         self.actionNextDiff.setEnabled(bool(self._diffs))
         self.actionPrevDiff.setEnabled(bool(self._diffs))
 
+        lexer = self.sci.lexer()
+
+        if lexer:
+            font = self.sci.lexer().font(0)
+        else:
+            font = self.sci.font()
+
+        fm = QFontMetrics(font)
+        maxWidth = fm.maxWidth()
+        lines = self.sci.text().split('\n')
+        widths = [fm.width(line) + maxWidth for line in lines]
+        self.maxWidth = max(widths)
+        self.updateScrollBar()
+
     #
     # These four functions are used by Shift+Cursor actions in revdetails
     #
@@ -674,6 +691,15 @@ class HgFileView(QFrame):
                 add(name, func)
         menu.exec_(point)
 
+    def resizeEvent(self, event):
+        super(HgFileView, self).resizeEvent(event)
+        self.updateScrollBar()
+
+    def updateScrollBar(self):
+        sbWidth = self.sci.verticalScrollBar().width()
+        scrollWidth = self.maxWidth + sbWidth - self.sci.width()
+        self.sci.showHScrollBar(scrollWidth > 0)
+        self.sci.horizontalScrollBar().setRange(0, scrollWidth)
 
 class AnnotateView(qscilib.Scintilla):
     'QScintilla widget capable of displaying annotations'
