@@ -49,7 +49,11 @@ class MessageEntry(qscilib.Scintilla):
         self.setContextMenuPolicy(Qt.CustomContextMenu)
         self.customContextMenuRequested.connect(self.menuRequested)
 
-
+    def setText(self, text):
+        result = super(MessageEntry, self).setText(text)
+        self.setDefaultEolMode()
+        return result
+        
     def menuRequested(self, point):
         line = self.lineAt(point)
         point = self.viewport().mapToGlobal(point)
