@@ -92,7 +92,12 @@ class RejectsDialog(QDialog):
         self.rejectbrowser.loadSettings(s, 'rejects/rejbrowse')
 
         f = QFile(path)
-        f.open(QIODevice.ReadOnly)
+        if not f.open(QIODevice.ReadOnly):
+            qtlib.ErrorMsgBox(_('Unable to merge rejects'),
+                              _("Can't read this file (maybe deleted)"))
+            self.hide()
+            QTimer.singleShot(0, self.reject)
+            return
         earlybytes = f.readData(4096)
         if '\0' in earlybytes:
             qtlib.ErrorMsgBox(_('Unable to merge rejects'),
@@ -131,7 +136,7 @@ class RejectsDialog(QDialog):
         QTimer.singleShot(0, lambda: self.chunklist.setCurrentRow(0))
 
     def menuRequested(self, point):
-        point = self.editor.mapToGlobal(point)
+        point = self.editor.viewport().mapToGlobal(point)
         return self.editor.createStandardContextMenu().exec_(point)
 
     def updateChunkList(self):
@@ -248,7 +253,7 @@ class RejectBrowser(qscilib.Scintilla):
         self.setLexer(lexer)
 
     def menuRequested(self, point):
-        point = self.mapToGlobal(point)
+        point = self.viewport().mapToGlobal(point)
         return self.createStandardContextMenu().exec_(point)
 
     def showChunk(self, lines):
