@@ -52,7 +52,7 @@ class WctxActions(QObject):
         make(_('Copy path'), copyPath, frozenset('MARC?!IS'), '')
         make(_('View missing'), viewmissing, frozenset('R!'))
         allactions.append(None)
-        make(_('&Revert...'), revert, frozenset('MAR!'), 'hg-revert')
+        make(_('&Revert...'), revert, frozenset('SMAR!'), 'hg-revert')
         make(_('&Add'), add, frozenset('R'), 'fileadd')
         allactions.append(None)
         make(_('File History'), log, frozenset('MARC!'), 'hg-log')
@@ -62,8 +62,6 @@ class WctxActions(QObject):
         make(_('&Add'), add, frozenset('I?'), 'fileadd')
         if 'largefiles' in self.repo.extensions():
             make(_('Add &Largefiles...'), addlf, frozenset('I?'))
-        elif 'kbfiles' in self.repo.extensions():
-            make(_('Add &Bfiles'), addlf, frozenset('I?'))
         make(_('&Detect Renames...'), guessRename, frozenset('A?!'),
              'detect_rename')
         make(_('&Ignore...'), ignore, frozenset('?'), 'ignore')
@@ -306,42 +304,22 @@ def forget(parent, ui, repo, files):
     return True
 
 def add(parent, ui, repo, files):
-    haslf = 'largefiles' in repo.extensions()
-    haskbf = 'kbfiles' in repo.extensions()
-    if haslf or haskbf:
-        result = lfprompt.promptForLfiles(parent, ui, repo, files, haskbf)
+    if 'largefiles' in repo.extensions():
+        result = lfprompt.promptForLfiles(parent, ui, repo, files)
         if not result:
             return False
         files, lfiles = result
-        for name, module in extensions.extensions():
-            if name == 'largefiles':
-                override_add = module.overrides.override_add
-                if files:
-                    override_add(commands.add, ui, repo, *files)
-                if lfiles:
-                    override_add(commands.add, ui, repo, large=True, *lfiles)
-                return True
-            if name == 'kbfiles':
-                override_add = module.bfsetup.override_add
-                if files:
-                    override_add(commands.add, ui, repo, *files)
-                if lfiles:
-                    override_add(commands.add, ui, repo, bf=True, *lfiles)
-                return True
-    commands.add(ui, repo, *files)
+        if files:
+            commands.add(ui, repo, normal=True, *files)
+        if lfiles:
+            commands.add(ui, repo, lfsize='', normal=False, large=True, *lfiles)
+    else:
+        commands.add(ui, repo, *files)
     return True
 
 def addlf(parent, ui, repo, files):
-    for name, module in extensions.extensions():
-        if name == 'largefiles':
-            override_add = module.overrides.override_add
-            override_add(commands.add, ui, repo, large=True, *files)
-            return True
-        if name == 'kbfiles':
-            override_add = module.bfsetup.override_add
-            override_add(commands.add, ui, repo, bf=True, *files)
-            return True
-    return False
+    commands.add(ui, repo, lfsize='', normal=None, large=True, *files)
+    return True
 
 def guessRename(parent, ui, repo, files):
     from tortoisehg.hgqt.guess import DetectRenameDialog
