@@ -73,12 +73,14 @@ if os.name == 'nt':
                 pass
         return os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
 
+    def is_unc_path(path):
+        unc, rest = os.path.splitunc(path)
+        return bool(unc)
+
     def is_on_fixed_drive(path):
-        if hasattr(os.path, 'splitunc'):
-            unc, rest = os.path.splitunc(path)
-            if unc:
-                # All UNC paths (\\host\mount) are considered not-fixed
-                return False
+        if is_unc_path(path):
+            # All UNC paths (\\host\mount) are considered not-fixed
+            return False
         drive, remain = os.path.splitdrive(path)
         if drive:
             return win32file.GetDriveType(drive) == win32file.DRIVE_FIXED
@@ -128,6 +130,9 @@ else: # Not Windows
         """
         return False
 
+    def is_unc_path(path):
+        return False
+
     def is_on_fixed_drive(path):
         return True
 
